/**
 * @file
 * Behaviors for initializing the Active Tags Autocomplete widget.
 */

(function ($, Drupal, drupalSettings, activeTags, debounce, JSON, Sortable, once) {

  /**
   * Initializes the Active Tags autocomplete widget for the input field.
   *
   * @param {Element} input
   *   The input element for the Active Tags widget.
   */
  function initAutocomplete(input) {
    // Set the context for the input.
    this.input = input;

    // Retrieve settings specific to the current autocomplete instance.
    const autocompleteSettings = drupalSettings.activeTagsAutocomplete[$(input).attr("id")];

    // Set local variables for identifier and cardinality.
    const identifier = input.dataset.identifier;
    const cardinality = parseInt(input.dataset.cardinality, 10) || Infinity;

    // Check the input has specific classes for auto-create and bubble styles.
    const autoCreate = $(input).hasClass('active-tags--auto-create');
    const bubbleStyle = $(input).hasClass('active-tags__tag__bubble');

    // Determine the appropriate message for no match scenarios.
    const noMatchMessage = autoCreate
      ? autocompleteSettings.not_exist_message
      : autocompleteSettings.not_found_message;

    // Define the template for no match dropdown items.
    const dropdownItemNoMatch = (data) =>
      !isTagLimitReached(cardinality, identifier) ?
        `<div class='${this.activeTags.settings.classNames.dropdownItem} active-tags__dropdown__item--no-match' value="noMatch" tabindex="0" role="option">
          <p>${Drupal.t(noMatchMessage, {"@term": data.value})}</p>
        </div>`
        : '';

    // Initialize the Active Tags widget.
    this.activeTags = Drupal.activeTags(input, {
      dropdown: {
        enabled: parseInt(input.dataset.suggestionsDropdown, 10),
        highlightFirst: true,
        fuzzySearch: !!parseInt(input.dataset.matchOperator, 10),
        maxItems: parseInt(input.dataset.maxItems, 10) || Infinity,
        closeOnSelect: true,
        searchKeys: ['label'],
        mapValueTo: 'label',
      },
      templates: {
        tag: tagTemplate,
        dropdownItem: function(tagData) {
          return suggestionItemTemplate.call(this, tagData, cardinality, identifier);
        },
        dropdownFooter: function(tagData) {
          return suggestionFooterTemplate.call(this, tagData, cardinality, identifier);
        },
        dropdownItemNoMatch: dropdownItemNoMatch,
      },
      classNames: {
        dropdown: bubbleStyle ? 'active-tags__tag__bubble active-tags__dropdown' : 'active-tags__dropdown',
      },
      whitelist: [],
      placeholder: parseInt(input.dataset.placeholder, 10),
      tagTextProp: 'label',
      editTags: false,
      maxTags: cardinality > 0 ? cardinality : Infinity,
    });

    /**
     * Binds Sortable to Active Tags main element
     * and specifies draggable items.
     */
    Sortable.create(this.activeTags.DOM.scope, {
      draggable: `.${this.activeTags.settings.classNames.tag}:not(active-tags__input)`,
      forceFallback: true,
      onEnd: () => {
        // Update Active Tags value according to the re-ordered nodes.
        document.onselectstart = null;
        this.activeTags.updateValueByDOMTags();
      },
    });

    // Active tags input event.
    const onInput = Drupal.debounce((e) => {
      const { value } = e.detail;
      handleAutocomplete.call(this, value, this.activeTags.value.map((item) => item.entity_id), identifier, cardinality);
    }, 300);

    // Active tags change event.
    const onChange = Drupal.debounce(() => {
      // Configure settings for Active Tags.
      this.activeTags.settings.enforceWhitelist =
        isTagLimitReached(cardinality, identifier) && cardinality > 1
          ? false
          : !autoCreate;
      this.activeTags.settings.skipInvalid = isTagLimitReached(cardinality, identifier)
        ? false
        : autoCreate;
    }, 300);

    // Input event (when a tag is being typed/edited.
    // e.detail exposes value, inputElm & isValid).
    this.activeTags.on('input', onInput);
    // Change event (any change to the value has occurred.
    // e.detail.value callback listener argument is a String).
    this.activeTags.on('change', onChange);

    // Trigger the onChange event to apply initial settings.
    onChange();

    // Handle dropdown suggestions enabled scenario.
    if (!this.activeTags.settings.dropdown.enabled) {
      const tagsElement = document.querySelector(`.${identifier}`);
      if (tagsElement) {
        tagsElement.classList.add('active-tags-select');
        document.addEventListener('click', (e) => {
          if (e.target.classList.contains('active-tags__input') && e.target.closest('.active-tags-widget')) {
            handleClickEvent.call(this, e, identifier, cardinality);
          }
        });
      } else {
        console.warn(`Element with identifier ${identifier} not found.`);
      }
    }

    // Add an event listener to handle image loading errors.
    document.addEventListener('DOMContentLoaded', () => {
      const images = document.querySelectorAll('.active-tags__info-label-img');
      images.forEach(img => {
        img.addEventListener('error', () => {
          img.style.visibility = 'hidden';
        });
      });
    });
  }

  /**
   * Initialize the Active Tags widget for the select element.
   *
   * @param {Element} select
   *   The select element for the Active Tags widget.
   */
  function initSelect(select) {
    // Set the select element to the instance's select property.
    this.select = select;
    this.mode = select.dataset.mode;

    // Retrieve settings specific to the current autocomplete instance.
    const selectSettings = drupalSettings.activeTagsSelect[$(select).attr("id")];

    // Assign the identifier from the dataset of the select element
    // to the instance's identifier property.
    const identifier = select.dataset.identifier;

    // Parse the cardinality value from the dataset, or default
    // to Infinity if not present.
    const cardinality = parseInt(select.dataset.cardinality, 10) || Infinity;

    // Destructure additional dataset properties for matchOperator,
    // matchLimit, and placeholder.
    const { matchOperator, matchLimit, placeholder } = select.dataset;

    // Check if the select element has the auto-create class.
    const autoCreate = $(select).hasClass('active-tags--auto-create');

    // Determine the appropriate message for no match scenarios.
    const noMatchMessage = autoCreate
      ? selectSettings.not_exist_message
      : selectSettings.not_found_message;

    // Define the template for no match dropdown items.
    const dropdownItemNoMatch = (data) =>
      !isTagLimitReached(cardinality, identifier)
        ? `<div class="${this.activeTagsSelect.settings.classNames.dropdownItem} active-tags__dropdown__item--no-match"
               value="noMatch"
               tabindex="0"
               role="option">
            <p>${Drupal.t(noMatchMessage, {"@term": data.value})}</p>
          </div>`
        : '';

    // Initialize arrays to store options and selected options.
    const options = [];
    const selected = [];

    // Iterate over each option element in the select element.
    [...select.options].forEach((option) => {
      // Add each option to the options array with its value and text.
      options.push({ value: option.value, text: option.text });

      // If the option is selected, add it to the selected array.
      if (option.selected) {
        selected.push({ value: option.value, text: option.text });
      }
    });

    // Insert an input element to attach Active Tags.
    const input = document.createElement('input');
    input.setAttribute('class', select.getAttribute('class'));
    input.value = JSON.stringify(selected);
    select.before(input);

    // Initialize the Active Tags widget with the provided configuration.
    this.activeTagsSelect = Drupal.activeTags(input, {
      dropdown: {
        enabled: 0,
        fuzzySearch: !!parseInt(matchOperator, 10),
        maxItems: matchLimit || Infinity,
        highlightFirst: true,
        searchKeys: ['text'],
        mapValueTo: 'text',
      },
      templates: {
        tag: tagSelectTemplate,
        dropdownItem: function(tagData) {
          return dropdownItemTemplate.call(this, tagData, cardinality, identifier);
        },
        dropdownFooter: function(tagData) {
          return suggestionFooterSelectTemplate.call(this, tagData, cardinality, identifier);
        },
        dropdownItemNoMatch: dropdownItemNoMatch,
      },
      classNames: {
        input: 'active-tags__input-select',
      },
      whitelist: options,
      enforceWhitelist: true,
      editTags: !!this.mode,
      maxTags: cardinality > 0 ? cardinality : Infinity,
      tagTextProp: 'text',
      placeholder,
    });

    // Remove active-tags--select class to keep Active Tags styles.
    if (this.mode) {
      const tagsElement = document.querySelector(`.${identifier}`);
      tagsElement.classList.remove('active-tags--select');
    }

    // Bind Sortable to Active Tags main element.
    Sortable.create(this.activeTagsSelect.DOM.scope, {
      draggable: `.${this.activeTagsSelect.settings.classNames.tag}:not(active-tags__input-select)`,
      forceFallback: true,
      onEnd: () => {
        Array.from(
          this.activeTagsSelect.DOM.scope.querySelectorAll(
            `.${this.activeTagsSelect.settings.classNames.tag}`
          )
        ).forEach((tag) => {
          const value = tag.getAttribute('value');
          const option = select.querySelector(`option[value="${value}"]`);
          if (option) {
            select.removeChild(option);
            select.appendChild(option);
            option.selected = tag.classList.contains('active-tags__tag');
          }
        });
      },
    });

    // Listen to add tag event.
    this.activeTagsSelect.on('add', (e) => {
      const { value } = e.detail.data;
      const option = select.querySelector(`option[value="${value}"]`);
      if (option) {
        select.removeChild(option);
        select.appendChild(option);
        option.selected = true;
      }
    });

    // Listen to remove tag event.
    this.activeTagsSelect.on('remove', (e) => {
      const { value } = e.detail.data;
      const option = select.querySelector(`option[value="${value}"]`);
      if (option) {
        option.selected = false;
      }
    });
  }

  /**
   * Handles autocomplete functionality for the input field using Active Tags.
   *
   * @param {string} value
   *   The current value of the input field.
   * @param {string[]} selectedEntities
   *   An array of selected entities.
   * @param {string} identifier
   *   The identifier for the active-tags widget.
   * @param {number} cardinality
   *   The maximum number of tags allowed.
   */
  function handleAutocomplete(value, selectedEntities, identifier, cardinality) {
    // Set the controller and whitelist for the active tags.
    let controller = this.controller;
    this.activeTags.whitelist = null;

    // Abort previous controller if exists.
    if (controller) {
      controller.abort();
    }
    controller = new AbortController();
    this.controller = controller;

    // Create Loading text markup if identifier is present.
    if (identifier) {
      createLoadingTextMarkup(identifier);
    }

    // Show loading animation if value is not empty.
    value !== '' ? this.activeTags.loading(true) : this.activeTags.loading(false);

    // Fetch the autocomplete data.
    fetch(
      `${$(this.input).attr('data-autocomplete-url')}?q=${encodeURIComponent(value)}&selected=${selectedEntities}`,
      { signal: controller.signal }
    )
      .then((res) => res.json())
      .then((newWhitelist) => {
        const newWhitelistData = [];

        // Process the fetched data and create new whitelist data.
        newWhitelist.forEach((current) => {
          newWhitelistData.push({
            value: current.entity_id,
            entity_id: current.entity_id,
            info_label: current.info_label,
            label: current.label,
            editable: current.editable,
            ...current.attributes,
          });
        });

        // Update the whitelist if data exists.
        if (newWhitelistData) {
          this.activeTags.whitelist = newWhitelistData;

          // Render the suggestion dropdown.
          this.activeTags.loading(false).dropdown.show(value);

          // Remove Loading text markup if identifier is present.
          if (identifier) {
            removeLoadingTextMarkup(identifier);
          }
        }

        // Show dropdown suggestion if tag limit is not reached.
        if (!isTagLimitReached(cardinality, identifier)) {
          this.activeTags.dropdown.show();
        }
      })
      .catch((error) => {
        console.error('Error fetching data:', error);
      });
  }

  /**
   * Handles click events on active tags input for autocomplete.
   *
   * @param {Event} e
   *   The click event object.
   * @param {string} identifier
   *   The identifier for the active-tags widget.
   * @param {number} cardinality
   *   The maximum number of tags allowed.
   */
  function handleClickEvent(e, identifier, cardinality) {
    // Check if the clicked target is the active tags input.
    const isActiveTagsInput = e.target.classList.contains('active-tags__input');
    // Check if the target is within the desired container.
    const isDesiredContainer = e.target.closest(`.${identifier}`);

    // Trigger autocomplete if conditions are met.
    if (isActiveTagsInput && isDesiredContainer) {
      handleAutocomplete(
        '',
        this.activeTags.value.map((item) => item.entity_id),
        identifier,
        cardinality,
      );
    }
  }

  /**
   * Checks if the tag limit has been reached.
   *
   * @param {number} cardinality
   *   The maximum number of tags allowed.
   * @param {string} identifier
   *   The identifier for the active-tags widget.
   *
   * @return {boolean}
   *   True if the tag limit has been reached, otherwise false.
   */
  function isTagLimitReached(cardinality, identifier) {
    // Check if the number of selected tags is greater than
    // or equal to the cardinality.
    /*console.log(cardinality);
    console.log(identifier);
    console.log(countSelectedTags(identifier));
    console.log(cardinality > 0 && countSelectedTags(identifier) >= cardinality);*/
    return cardinality > 0 && countSelectedTags(identifier) >= cardinality;
  }

  /**
   * Counts the number of selected tags.
   *
   * @param {string} identifier
   *   The identifier for the active-tags widget.
   *
   * @return {number}
   *   The number of selected tags.
   */
  function countSelectedTags(identifier) {
    // Select the element with the provided identifier.
    const tagsElement = document.querySelector(`.${identifier}`);

    // If the tags element exists, count the number of
    // tags with the class 'active-tags__tag'.
    if (tagsElement) {
      const tagElements = tagsElement.querySelectorAll('.active-tags__tag');
      return tagElements.length;
    }

    // If the tags element does not exist, return 0.
    return 0;
  }

  /**
   * Creates loading text markup if it doesn't exist.
   *
   * @param {string} identifier
   *   The identifier for the active-tags widget.
   */
  function createLoadingTextMarkup(identifier) {
    // Select the element with the provided identifier.
    const tagsElement = document.querySelector(`.${identifier}`);

    // Check if tagsElement exists.
    if (!tagsElement) return;

    // Check if the element with class 'loading-text' exists.
    const existingLoadingText = tagsElement.querySelector('.active-tags--loading-text');

    // If the loading text element doesn't exist, create and append it.
    if (!existingLoadingText) {
      const loadingText = document.createElement('div');
      loadingText.className = 'active-tags--loading-text hidden';
      loadingText.textContent = 'Loading...';
      tagsElement.appendChild(loadingText);
    }
  }

  /**
   * Removes loading text markup.
   *
   * @param {string} identifier
   *   The identifier for the active-tags widget.
   */
  function removeLoadingTextMarkup(identifier) {
    // Select the element with the provided identifier.
    const tagsElement = document.querySelector(`.${identifier}`);

    // If the tags element exists, find and remove the loading text element.
    if (tagsElement) {
      const loadingText = tagsElement.querySelector('.active-tags--loading-text');
      if (loadingText) {
        loadingText.remove();
      }
    }
  }

  /**
   * Generates HTML markup for an entity id.
   *
   * @param {string} entityId
   *   The entity id.
   *
   * @return {string}
   *   The entity id markup HTML.
   */
  function entityIdMarkup(entityId) {
    // Ensure 'input' is defined and accessible in the context.
    if (this.input && parseInt(this.input.dataset.showEntityId, 10) && entityId) {
      return `
      <div class="active-tags__tag__with-entity-id">
        <div class="active-tags__tag__entity-id-wrap">
          <span class="active-tags__tag__entity-id">${entityId}</span>
        </div>
      </div>
    `;
    }
    return '';
  }

  /**
   * Generates HTML markup for an info label.
   *
   * @param {string} infoLabel
   *   The info label information.
   *
   * @return {string}
   *   The info label markup HTML.
   */
  function infoLabelMarkup(infoLabel) {
    // Return an empty string if infoLabel is not provided.
    if (!infoLabel) {
      return '';
    }

    // Generate the markup based on whether the info label is an image or plain text.
    if (validImgSrc(infoLabel) || containsSVG(infoLabel)) {
      const imgMarkup = validImgSrc(infoLabel)
        ? `<img src="${infoLabel}" class="active-tags__info-label-img">`
        : infoLabel;

      return `
      <div class="active-tags__tag__info-label-wrap">
        <div class="active-tags__tag__info-label-image">
          ${imgMarkup}
        </div>
      </div>
    `;
    } else {
      return `
      <div class="active-tags__tag__info-label-wrap">
        <span class="active-tags__tag__info-label">${infoLabel}</span>
      </div>
    `;
    }
  }

  /**
   * Generates HTML markup for a tag.
   *
   * @param {string} tagLabel
   *   The label for the tag.
   * @param {string} tagInfoLabel
   *   The info label information.
   * @param {string} tagEntityId
   *   The entity id.
   *
   * @return {string}
   *   The tag markup HTML.
   */
  function tagMarkup(tagLabel, tagInfoLabel, tagEntityId) {
    return `
    <div class="active-tags__tag__items">
      ${tagEntityId}
      <span class="${tagEntityId ? 'active-tags__tag__text-with-entity-id' : 'active-tags__tag__text'}">${tagLabel}</span>
      ${tagInfoLabel}
    </div>
  `;
  }

  /**
   * Generates HTML markup for a tag based on the provided tagData.
   *
   * Creates the HTML structure for a tag in the Active Tags widget using
   * the given tagData, including label, entity ID, and additional attributes.
   *
   * @param {Object} tagData
   *   Data for the tag, including value, entity_id, class, etc.
   *
   * @return {string}
   *   The HTML markup for the generated tag.
   */
  function tagTemplate(tagData) {
    // Avoid 'undefined' values on paste event.
    const label = tagData.label ?? tagData.value;

    // Generate the tag HTML markup.
    return `
    <tag title="${tagData.label}"
         contenteditable="false"
         spellcheck="false"
         tabIndex="-1"
         class="active-tags__tag ${tagData.class ? tagData.class : ''}"
         ${this.getAttributes(tagData)}>
      <x title="Remove ${tagData.label}"
         class="active-tags__tag__removeBtn"
         role="button"
         aria-label="remove ${tagData.label} tag"
         tabindex="0">
      </x>
      ${tagMarkup(
      label,
      infoLabelMarkup(tagData.info_label),
      entityIdMarkup(tagData.entity_id)
    )}
    </tag>
  `;
  }

  /**
   * Generates HTML markup for a tag based on the provided tagData.
   *
   * @param {Object} tagData
   *   Data for the tag, including value, text, class, etc.
   *
   * @return {string}
   *   HTML markup for the generated tag.
   */
  function tagSelectTemplate(tagData) {
    return `
    <tag title="${tagData.text}"
         contenteditable="false"
         spellcheck="false"
         tabIndex="-1"
         class="active-tags__tag ${tagData.class ? tagData.class : ''}"
         ${this.getAttributes(tagData)}>
      <x class="active-tags__tag__removeBtn"
         role="button"
         aria-label="remove tag"
         tabIndex="0">
      </x>
      <div class="active-tags__tag__items">
        <span class="active-tags__tag__text">${tagData.text}</span>
      </div>
    </tag>
  `;
  }

  /**
   * Generates HTML markup for a dropdown item based on the provided tagData.
   *
   * @param {Object} tagData
   *   Data for the tag, including value, text, etc.
   * @param {number} cardinality
   *   The maximum number of tags allowed.
   * @param {string} identifier
   *   The identifier for the active-tags widget.
   *
   * @return {string}
   *   HTML markup for the generated dropdown item.
   */
  function dropdownItemTemplate(tagData, cardinality, identifier) {
    // Retrieve class names from the settings.
    const { classNames } = this.settings;

    // Check if the tag limit is not reached or if the mode is active.
    if (!isTagLimitReached(cardinality, identifier) || this.mode) {
      const dropdownItemClass = classNames.dropdownItem;
      const highlightedText = highlightMatchingLetters(tagData.text, this.state.inputText);

      // Return the HTML markup for the dropdown item.
      return `
      <div class="${dropdownItemClass}"
           value="${tagData.value}"
           tabindex="0"
           role="option">
        <div class="active-tags__dropdown__item-highlighted">${highlightedText}</div>
      </div>
    `;
    }

    // Return an empty if the tag limit is reached and mode is not active.
    return '';
  }

  /**
   * Generates the HTML for a suggestion item.
   *
   * Constructs the HTML for a suggestion in the Active Tags dropdown based on
   * the provided tagData, which includes the item’s label, class, and other
   * attributes.
   *
   * @param {Object} tagData
   *   Data representing the suggestion item.
   * @param {number} cardinality
   *   The maximum number of tags allowed.
   * @param {string} identifier
   *   The identifier for the active-tags widget.
   *
   * @return {string}
   *   HTML template for the suggestion item.
   */
  function suggestionItemTemplate(tagData, cardinality, identifier) {
    // Returns suggestion item when the field cardinality is unlimited or
    // field cardinality is bigger than the number of selected tags.
    if (!isTagLimitReached(cardinality, identifier)) {
      return `
      <div ${this.getAttributes(tagData)}
           class="active-tags__dropdown__item ${tagData.class ? tagData.class : ''}"
           tabindex="0"
           role="option">
        <div class="active-tags__dropdown__item-highlighted">
          ${highlightMatchingLetters(tagData.label, this.state.inputText)}
        </div>
        ${infoLabelMarkup(tagData.info_label)}
      </div>
    `;
    }
    // Return an empty string if the tag limit is reached.
    return '';
  }

  /**
   * Generates the HTML for the suggestion footer.
   *
   * Constructs the HTML for the footer in the Active Tags dropdown based on
   * the provided cardinality and identifier, displaying a message when the tag
   * limit has been reached.
   *
   * @param {Object} tagData
   *   Data representing the suggestion item.
   * @param {number} cardinality
   *   The maximum number of tags allowed.
   * @param {string} identifier
   *   The identifier for the active-tags widget.
   *
   * @return {string}
   *   HTML template for the suggestion footer.
   */
  function suggestionFooterTemplate(tagData, cardinality, identifier) {
    // Returns empty dropdown footer when field cardinality is unlimited or
    // field cardinality is bigger than the number of selected tags.
    if (isTagLimitReached(cardinality, identifier)) {
      return `
      <footer data-selector="active-tags-suggestions-footer" class="${this.settings.classNames.dropdownFooter}">
        <p>${Drupal.t(drupalSettings.activeTagsAutocomplete[$(input).attr("id")].limit_tag_message, {"@cardinality": cardinality})}</p>
      </footer>
    `;
    }
    // Return an empty string if the tag limit is not reached.
    return '';
  }

  /**
   * Generates the HTML for the suggestion footer.
   *
   * Constructs the HTML for the footer in the Active Tags dropdown based on
   * the provided cardinality and identifier, displaying a message when the tag
   * limit has been reached.
   *
   * @param {Object} tagData
   *   Data representing the suggestion item.
   * @param {number} cardinality
   *   The maximum number of tags allowed.
   * @param {string} identifier
   *   The identifier for the active-tags widget.
   *
   * @return {string}
   *   The HTML template for the suggestion footer.
   */
  function suggestionFooterSelectTemplate(tagData, cardinality, identifier) {
    // Extract classNames from the settings.
    const { classNames } = this.settings;

    // Returns empty dropdown footer when field cardinality is unlimited or
    // field cardinality is bigger than the number of selected tags.
    if (isTagLimitReached(cardinality, identifier) && !this.mode) {
      return `
      <footer data-selector="active-tags-suggestions-footer" class="${classNames.dropdownFooter}">
        <p>${Drupal.t(drupalSettings.activeTagsSelect[$(select).attr("id")].limit_tag_message, {"@cardinality": cardinality})}</p>
      </footer>
    `;
    }

    // Return an empty string if the tag limit is not reached or mode is set.
    return '';
  }

  /**
   * Highlights matching letters in a string.
   *
   * This function takes an input string and a search term, then highlights
   * all occurrences of the search term within the input string by wrapping
   * them in <strong> tags. This is useful for emphasizing matched terms
   * in search results or autocomplete suggestions.
   *
   * @param {string} inputTerm
   *   The input string where the search is performed.
   * @param {string} searchTerm
   *   The term to highlight within the input string.
   *
   * @return {string}
   *   The input string with matching letters wrapped in <strong> tags.
   */
  function highlightMatchingLetters(inputTerm, searchTerm) {
    // Escape special characters in the search term to
    // prevent issues with RegExp.
    const escapedSearchTerm = searchTerm.replace(/[.*+?^${}()|[\]\\]/g, '\\$&');

    // Create a regular expression to match the search term globally
    // and case insensitively.
    const regex = new RegExp(`(${escapedSearchTerm})`, 'gi');

    // Check if the search term is empty and return
    // the original string if it is.
    if (!escapedSearchTerm) {
      return inputTerm;
    }

    // Replace matching letters with the same letters wrapped in <strong> tags.
    return inputTerm.replace(regex, '<strong>$1</strong>');
  }

  /**
   * Checks if a given string contains SVG elements.
   *
   * This function takes a string and checks if it contains any SVG elements by
   * creating a temporary DOM element and setting its innerHTML to the provided
   * string. It then queries the DOM element for any SVG elements.
   *
   * @param {string} infoLabel
   *   The string to be checked for SVG elements.
   *
   * @return {boolean}
   *   Returns true if the string contains SVG elements, otherwise false.
   */
  function containsSVG(infoLabel) {
    // Check if infoLabel is defined and is a string.
    if (infoLabel && typeof infoLabel === 'string') {
      // Create a temporary div element.
      const tempDiv = document.createElement('div');
      // Set the innerHTML of the div to the content of infoLabel.
      tempDiv.innerHTML = infoLabel;
      // Check if the div contains any SVG elements.
      const svgElements = tempDiv.querySelectorAll('svg');
      // Return true if any SVG elements are found, false otherwise.
      return svgElements.length > 0;
    }
    // Return false if infoLabel is not defined or not a string.
    return false;
  }

  /**
   * Checks if the info label is a valid image source.
   *
   * This function uses a regular expression to test if the provided string is
   * a valid URL for an image file. Supported image extensions include jpg,
   * jpeg, png, gif, bmp, svg, and webp.
   *
   * @param {string} infoLabel
   *   The info label input.
   *
   * @return {boolean}
   *   True if the info label is a valid image source.
   */
  function validImgSrc(infoLabel) {
    // Define the regular expression pattern for a valid image URL.
    const pattern = new RegExp(
      '^(https?:\\/\\/)?' +
      '((([a-z\\d]([a-z\\d-]*[a-z\\d])*)\\.)+[a-z]{2,}|' +
      '((\\d{1,3}\\.){3}\\d{1,3}))' +
      '(\\:\\d+)?' +
      '(\\/[-a-z\\d%_.~+]*)*' +
      '(\\.(?:jpg|jpeg|png|gif|bmp|svg|webp))' +
      '(\\?[;&a-z\\d%_.~+=-]*)?' +
      '(\\#[-a-z\\d_]*)?$',
      'i'
    );
    // Test the infoLabel against the pattern and return the result.
    return !!pattern.test(infoLabel);
  }

  /**
   * Attaches the behavior for initializing Active Tags Autocomplete widget.
   *
   * @type {Drupal~behavior}
   *
   * @prop {Drupal~behaviorAttach} attach
   *   Attaches the behavior for the Active Tags Autocomplete widget.
   */
  Drupal.behaviors.activeTagsWidgets = {
    input: null,
    select: null,
    mode: null,
    controller: null,
    activeTags: null,
    activeTagsSelect: null,
    attach: function (context, settings) {
      // Initialize Active Tags on specific elements.
      once('active-tags-widget', 'input.active-tags-widget', context).forEach(
        initAutocomplete,
      );
      once('active-tags-select-widget', 'select.active-tags-select-widget', context).forEach(
        initSelect,
      );
    }
  };

})(jQuery, Drupal, drupalSettings, Drupal.activeTags, Drupal.debounce, window.JSON, Sortable, once);
